# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


from migtools import migrremote
import unittest
import mock
import os

TEMP_FILE_NAME = "temp.xml"
CURRENT_DIRECTORY = os.getcwd() + "/"


class MigrremoteTest(unittest.TestCase):

    def setUp(self):
        super(MigrremoteTest, self).setUp()
        self.parser = migrremote.create_parser()
        # create a temp xml file
        with open(TEMP_FILE_NAME, "w+") as input_file:
            input_file.write("Dummy data")

    def test_empty_args(self):
        with self.assertRaises(SystemExit):
            self.parser.parse_args([])

    def test_invalid_args(self):

        # missing -o parameter
        with self.assertRaises(SystemExit):
            self.parser.parse_args(["-f", CURRENT_DIRECTORY + TEMP_FILE_NAME])

        # invalid -o parameter
        with self.assertRaises(SystemExit):
            self.parser.parse_args(["-f", CURRENT_DIRECTORY + TEMP_FILE_NAME,
                                    "-o", "k"])

    @mock.patch("migtools.migration.validate")
    def test_invoke_operation(self, mock_validate):

        mock_validate.return_value = "Dummy output"

        args = self.parser.parse_args(["-o", "v", "-f", CURRENT_DIRECTORY +
                                       TEMP_FILE_NAME])
        migrremote.invoke_operation(args.o, args)

        self.assertTrue(mock_validate.called)


if __name__ == "__main__":
    try:
        unittest.main()
    finally:
        # always delete the temp created file
        os.remove(TEMP_FILE_NAME)
