# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2016
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


from pvmcli.command.object_type import ObjectType
import pvmcli.help.help_shared_storage_pool as help_ssp
from pvmcli.util.i18n import _

from pypowervm.wrappers import storage as s


PRI_DISP_FIELD_LABELS = [_("Name"), _("Cap (GB)"), _("Free Space"),
                         _("OC Space")]


class SharedStoragePool(ObjectType):
    def __init__(self):
        super(SharedStoragePool, self).__init__()

        self.add_field_models([s.LU, s.PV])

        self.add_display_operation()

        self.setup_arguments()

    @property
    def model(self):
        return s.SSP

    def ssp_out_rows(self):
        out_rows = []
        for wrp in self.ec.wrappers:
            out_rows.append(
                [self.dfield_val(wrp, "SharedStoragePool.name"),
                 self.dfield_val(wrp, "SharedStoragePool.capacity"),
                 self.dfield_val(wrp, "SharedStoragePool.free_space"),
                 self.dfield_val(wrp, "SharedStoragePool.over_commit_space")])
        return out_rows

    def pri_output(self):
        po_table = self.resize_pri_out_table(PRI_DISP_FIELD_LABELS,
                                             self.ssp_out_rows())

        return "%(n_ttl)s\n%(n_tbl)s" % {
            'n_ttl': _("Shared Storage Pools"), 'n_tbl': po_table.get_string()}

    @property
    def help(self):
        return help_ssp.help_text
