# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2016, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import six

from pvmcli.command.object_type import ObjectType
import pvmcli.help.help_virtual_network as help_vnet
from pvmcli.util.i18n import _
import pvmcli.util.util as u

from pypowervm.wrappers import network as n
from pypowervm.wrappers import managed_system as ms_w


PRI_DISP_FIELD_LABELS = [_("VLAN"), _("VSwitch"), _("Tagged"),
                         _("VIOS"), _("Back Dev"), _("SEA")]


class VirtualNetwork(ObjectType):
    def __init__(self):
        super(VirtualNetwork, self).__init__()

        self.add_display_operation(parent_models=[ms_w.System])

        self.setup_arguments()

    def _bld_vlan_to_nbr_map(self):
        vlan_to_nbrs = {}
        all_nbr = n.NetBridge.wrap(
            self.ec.sm.adapter.read(root_type=ms_w.System.schema_type,
                                    root_id=self.ec.msys.uuid,
                                    child_type=n.NetBridge.schema_type))

        if not isinstance(all_nbr, list):
            all_nbr = [all_nbr]

        for nbr in all_nbr:
            for vlan in nbr.list_vlans():
                if six.text_type(vlan) in vlan_to_nbrs:
                    vlan_to_nbrs[six.text_type(vlan)].append(nbr)
                else:
                    vlan_to_nbrs[six.text_type(vlan)] = [nbr]

        return vlan_to_nbrs

    def _bld_out_rows(self, wrp, vlan):
        b_out_row = []
        b_out_rows = []
        found_vsw = u.vswitch_by_uri(self.ec.sm.adapter,
                                     wrp.associated_switch_uri)
        b_out_row.extend([vlan, found_vsw.name, u.bool_str(wrp.tagged)])
        vlan_to_nbrs = self._bld_vlan_to_nbr_map()

        if vlan in vlan_to_nbrs:
            for nbr in [nbrs for nbrs in vlan_to_nbrs[vlan]]:
                for sea in nbr.seas:
                    b_out_row.extend([u.vios_by_uri(self.ec.sm.adapter,
                                                    sea.vio_uri).name,
                                      sea.backing_device.dev_name,
                                      sea.dev_name])
                    b_out_rows.append(b_out_row)
                    b_out_row = ["", "", ""]
        else:
            b_out_row.extend(["", "", ""])
            b_out_rows.append(b_out_row)
        return b_out_rows

    def vnet_out_rows(self):
        out_rows = []
        vlan_to_wrp = {}
        for wrp in self.ec.wrappers:
            vlan_to_wrp[self.dfield_val(wrp, "VirtualNetwork.vlan")] = wrp

        for vlan in sorted(vlan_to_wrp):
            out_rows.extend(self._bld_out_rows(vlan_to_wrp[vlan], vlan))

        return out_rows

    @property
    def model(self):
        return n.VNet

    def pri_output(self):
        po_table = self.resize_pri_out_table(PRI_DISP_FIELD_LABELS,
                                             self.vnet_out_rows())

        return "%(n_ttl)s\n%(n_tbl)s" % {
            'n_ttl': _("Virtual Networks"), 'n_tbl': po_table.get_string()}

    @property
    def help(self):
        return help_vnet.help_text
