# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2016
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import six

from pvmcli.command.object_type import ObjectType
import pvmcli.help.help_volume_group as help_vg
from pvmcli.util.i18n import _
import pvmcli.util.util as u

from pypowervm.wrappers import storage as s
from pypowervm.wrappers import virtual_io_server as v


LV_SCHEMA = "LogicalVolume"

PRI_DISP_FIELD_LABELS = [_("VIOS"), _("VG Name"), _("VG Size (GB)"),
                         _("Free (GB)"), _("LV Name"), _("LV Size (GB)")]


def _get_out_rows(vios_name, vg):
    ret_rows = []
    out_row = [vios_name, vg.name, six.text_type(vg.capacity),
               six.text_type(vg.free_space)]
    if not vg.virtual_disks:
        out_row.extend(['', ''])
        ret_rows.append(out_row)
    else:
        for lv_wrp in vg.virtual_disks:
            out_row.extend([lv_wrp.name, six.text_type(lv_wrp.capacity)])
            ret_rows.append(out_row)
            out_row = ["", "", "", ""]

    return ret_rows


def vg_out_rows(obj_type, vg_wrps):
    out_rows = []
    vios_to_vg = {}

    for vg_wrp in vg_wrps:
        vios_name = u.vios_for_vg(obj_type.ec.sm.adapter, vg_wrp.href).name
        if vios_name in vios_to_vg:
            vios_to_vg[vios_name].append(vg_wrp)
        else:
            vios_to_vg[vios_name] = [vg_wrp]

    for vios_name in sorted(vios_to_vg):
        for vg in vios_to_vg[vios_name]:
            out_rows.extend(_get_out_rows(vios_name, vg))

    return out_rows


class VolumeGroup(ObjectType):
    def __init__(self):
        super(VolumeGroup, self).__init__()

        self.remap_schema(s.VDisk.schema_type, LV_SCHEMA)

        self.add_field_models([s.VMediaRepos, s.PV, s.VDisk, s.VOptMedia])

        self.add_display_operation(parent_models=[v.VIOS])

        self.setup_arguments()

    @property
    def model(self):
        return s.VG

    def pri_output(self):
        po_table = self.resize_pri_out_table(
            PRI_DISP_FIELD_LABELS, vg_out_rows(self, self.ec.wrappers))

        return "%(n_ttl)s\n%(n_tbl)s" % {
            'n_ttl': _("Volume Groups"), 'n_tbl': po_table.get_string()}

    @property
    def help(self):
        return help_vg.help_text
