# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2015, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import mock

import pvmcli.cli_parser as cli_parser
import pvmcli.command.virtual_io_server as vios_c
import pvmcli.help.help_virtual_io_server as help_vios
import pvmcli.tests.util.test_util as test_util
import pvmcli.util.exceptions as exc

import pypowervm.tests.test_utils.test_wrapper_abc as twa
import pypowervm.wrappers.base_partition as bp_w
from pypowervm.wrappers import managed_system as ms_w
import pypowervm.wrappers.network as n
import pypowervm.wrappers.storage as stor
import pypowervm.wrappers.virtual_io_server as vios_w


LIST_USAGE_HELP = (["usage: pvmctl [help | --help] VirtualIOServer list "
                    "{--object-id | -i} <VirtualIOServer field>=value "
                    "{args}"])


DELETE_USAGE_HELP = (["usage: pvmctl [help | --help] VirtualIOServer delete "
                      "{--object-id | -i} <VirtualIOServer field>=value "
                      "{args}"])

DISPLAY_FIELD_HELP = ([
    "--display-fields options:\n"
    "    AssociatedIOSlot.bus_grp_required\n"
    "    AssociatedIOSlot.description\n"
    "    AssociatedIOSlot.drc_index\n"
    "    AssociatedIOSlot.drc_name\n"
    "    AssociatedIOSlot.io_adapter\n"
    "    AssociatedIOSlot.pc_adpt_id\n"
    "    AssociatedIOSlot.pci_class\n"
    "    AssociatedIOSlot.pci_dev_id\n"
    "    AssociatedIOSlot.pci_mfg_id\n"
    "    AssociatedIOSlot.pci_rev_id\n"
    "    AssociatedIOSlot.pci_subsys_dev_id\n"
    "    AssociatedIOSlot.pci_subsys_vendor_id\n"
    "    AssociatedIOSlot.pci_vendor_id\n"
    "    AssociatedIOSlot.phys_loc\n"
    "    ClientAdapter.is_varied_on\n"
    "    ClientAdapter.loc_code\n"
    "    ClientAdapter.lpar_id\n"
    "    ClientAdapter.lpar_slot_num\n"
    "    ClientAdapter.side\n"
    "    ClientAdapter.vios_id\n"
    "    ClientAdapter.vios_slot_num\n"
    "    ClientAdapter.wwpns\n"
    "    DedicatedProcessorConfiguration.desired\n"
    "    DedicatedProcessorConfiguration.max\n"
    "    DedicatedProcessorConfiguration.min\n"
    "    EthernetBackingDevice.adapter_id\n"
    "    EthernetBackingDevice.dev_name\n"
    "    IOAdapter.description\n"
    "    IOAdapter.dev_name\n"
    "    IOAdapter.dev_type\n"
    "    IOAdapter.drc_name\n"
    "    IOAdapter.id\n"
    "    IOAdapter.phys_loc_code\n"
    "    IOAdapter.udid\n"
    "    IOAdapterChoice.description\n"
    "    IOAdapterChoice.dev_name\n"
    "    IOAdapterChoice.dev_type\n"
    "    IOAdapterChoice.drc_name\n"
    "    IOAdapterChoice.id\n"
    "    IOAdapterChoice.phys_loc_code\n"
    "    IOAdapterChoice.udid\n"
    "    LogicalUnit.capacity\n"
    "    LogicalUnit.cloned_from_udid\n"
    "    LogicalUnit.emulate_model\n"
    "    LogicalUnit.in_use\n"
    "    LogicalUnit.is_thin\n"
    "    LogicalUnit.lu_type\n"
    "    LogicalUnit.name\n"
    "    LogicalUnit.tag\n"
    "    LogicalUnit.udid\n"
    "    LogicalVolumeVirtualTargetDevice.lua\n"
    "    LogicalVolumeVirtualTargetDevice.name\n"
    "    PartitionCapabilities.io_dlpar\n"
    "    PartitionCapabilities.mem_dlpar\n"
    "    PartitionCapabilities.proc_dlpar\n"
    "    PartitionIOConfiguration.io_slots\n"
    "    PartitionIOConfiguration.max_virtual_slots\n"
    "    PartitionIOConfiguration.tagged_io\n"
    "    PartitionMemoryConfiguration.ame_enabled\n"
    "    PartitionMemoryConfiguration.current\n"
    "    PartitionMemoryConfiguration.desired\n"
    "    PartitionMemoryConfiguration.exp_factor\n"
    "    PartitionMemoryConfiguration.max\n"
    "    PartitionMemoryConfiguration.min\n"
    "    PartitionMemoryConfiguration.ppt_ratio\n"
    "    PartitionMemoryConfiguration.shared_enabled\n"
    "    PartitionProcessorConfiguration.dedicated_proc_cfg\n"
    "    PartitionProcessorConfiguration.has_dedicated\n"
    "    PartitionProcessorConfiguration.shared_proc_cfg\n"
    "    PartitionProcessorConfiguration.sharing_mode\n"
    "    PhysicalFibreChannelAdapter.description\n"
    "    PhysicalFibreChannelAdapter.dev_name\n"
    "    PhysicalFibreChannelAdapter.dev_type\n"
    "    PhysicalFibreChannelAdapter.drc_name\n"
    "    PhysicalFibreChannelAdapter.fc_ports\n"
    "    PhysicalFibreChannelAdapter.id\n"
    "    PhysicalFibreChannelAdapter.phys_loc_code\n"
    "    PhysicalFibreChannelAdapter.udid\n"
    "    PhysicalFibreChannelPort.loc_code\n"
    "    PhysicalFibreChannelPort.name\n"
    "    PhysicalFibreChannelPort.npiv_available_ports\n"
    "    PhysicalFibreChannelPort.npiv_total_ports\n"
    "    PhysicalFibreChannelPort.udid\n"
    "    PhysicalFibreChannelPort.wwpn\n"
    "    PhysicalVolume.avail_for_use\n"
    "    PhysicalVolume.capacity\n"
    "    PhysicalVolume.description\n"
    "    PhysicalVolume.emulate_model\n"
    "    PhysicalVolume.is_fc_backed\n"
    "    PhysicalVolume.loc_code\n"
    "    PhysicalVolume.name\n"
    "    PhysicalVolume.read_iops_limit\n"
    "    PhysicalVolume.state\n"
    "    PhysicalVolume.tag\n"
    "    PhysicalVolume.udid\n"
    "    PhysicalVolume.write_iops_limit\n"
    "    PhysicalVolumeVirtualTargetDevice.lua\n"
    "    PhysicalVolumeVirtualTargetDevice.name\n"
    "    ProfileIOSlot.bus_grp_required\n"
    "    ProfileIOSlot.description\n"
    "    ProfileIOSlot.drc_index\n"
    "    ProfileIOSlot.drc_name\n"
    "    ProfileIOSlot.io_adapter\n"
    "    ProfileIOSlot.pc_adpt_id\n"
    "    ProfileIOSlot.pci_class\n"
    "    ProfileIOSlot.pci_dev_id\n"
    "    ProfileIOSlot.pci_mfg_id\n"
    "    ProfileIOSlot.pci_rev_id\n"
    "    ProfileIOSlot.pci_subsys_dev_id\n"
    "    ProfileIOSlot.pci_subsys_vendor_id\n"
    "    ProfileIOSlot.pci_vendor_id\n"
    "    ProfileIOSlot.phys_loc\n"
    "    ProfileIOSlot.required\n"
    "    ServerAdapter.backing_dev_name\n"
    "    ServerAdapter.is_varied_on\n"
    "    ServerAdapter.loc_code\n"
    "    ServerAdapter.lpar_id\n"
    "    ServerAdapter.lpar_slot_num\n"
    "    ServerAdapter.map_port\n"
    "    ServerAdapter.name\n"
    "    ServerAdapter.side\n"
    "    ServerAdapter.udid\n"
    "    ServerAdapter.vios_id\n"
    "    ServerAdapter.vios_slot_num\n"
    "    SharedEthernetAdapter.addl_adpts\n"
    "    SharedEthernetAdapter.backing_device\n"
    "    SharedEthernetAdapter.configuration_state\n"
    "    SharedEthernetAdapter.control_channel\n"
    "    SharedEthernetAdapter.dev_name\n"
    "    SharedEthernetAdapter.ha_mode\n"
    "    SharedEthernetAdapter.primary_adpt\n"
    "    SharedEthernetAdapter.pvid\n"
    "    SharedEthernetAdapter.vio_uri\n"
    "    SharedProcessorConfiguration.desired_units\n"
    "    SharedProcessorConfiguration.desired_virtual\n"
    "    SharedProcessorConfiguration.max_units\n"
    "    SharedProcessorConfiguration.max_virtual\n"
    "    SharedProcessorConfiguration.min_units\n"
    "    SharedProcessorConfiguration.min_virtual\n"
    "    SharedProcessorConfiguration.pool_id\n"
    "    SharedProcessorConfiguration.uncapped_weight\n"
    "    SharedStoragePoolLogicalUnitVirtualTargetDevice.lua\n"
    "    SharedStoragePoolLogicalUnitVirtualTargetDevice.name\n"
    "    TaggedIO.alt_load_src\n"
    "    TaggedIO.console\n"
    "    TaggedIO.load_src\n"
    "    TrunkAdapter.associated_vswitch_uri\n"
    "    TrunkAdapter.dev_name\n"
    "    TrunkAdapter.has_tag_support\n"
    "    TrunkAdapter.loc_code\n"
    "    TrunkAdapter.pvid\n"
    "    TrunkAdapter.tagged_vlans\n"
    "    TrunkAdapter.trunk_pri\n"
    "    TrunkAdapter.varied_on\n"
    "    TrunkAdapter.vios_id\n"
    "    TrunkAdapter.virtual_slot_number\n"
    "    TrunkAdapter.vswitch_id\n"
    "    VirtualDisk.backstore_type\n"
    "    VirtualDisk.capacity\n"
    "    VirtualDisk.emulate_model\n"
    "    VirtualDisk.file_format\n"
    "    VirtualDisk.label\n"
    "    VirtualDisk.name\n"
    "    VirtualDisk.read_iops_limit\n"
    "    VirtualDisk.tag\n"
    "    VirtualDisk.udid\n"
    "    VirtualDisk.vdtype\n"
    "    VirtualDisk.vg_uri\n"
    "    VirtualDisk.write_iops_limit\n"
    "    VirtualFibreChannelMapping.backing_port\n"
    "    VirtualFibreChannelMapping.client_adapter\n"
    "    VirtualFibreChannelMapping.client_lpar_href\n"
    "    VirtualFibreChannelMapping.server_adapter\n"
    "    VirtualIOServer.allow_perf_data_collection\n"
    "    VirtualIOServer.assoc_sys_uuid\n"
    "    VirtualIOServer.avail_priority\n"
    "    VirtualIOServer.bootmode\n"
    "    VirtualIOServer.capabilities\n"
    "    VirtualIOServer.current_secure_boot\n"
    "    VirtualIOServer.disable_secure_boot\n"
    "    VirtualIOServer.env\n"
    "    VirtualIOServer.id\n"
    "    VirtualIOServer.io_adpts_for_link_agg\n"
    "    VirtualIOServer.io_config\n"
    "    VirtualIOServer.ip_addresses\n"
    "    VirtualIOServer.is_license_accepted\n"
    "    VirtualIOServer.is_mgmt_partition\n"
    "    VirtualIOServer.is_mover_service_partition\n"
    "    VirtualIOServer.is_service_partition\n"
    "    VirtualIOServer.keylock_pos\n"
    "    VirtualIOServer.media_repository\n"
    "    VirtualIOServer.mem_config\n"
    "    VirtualIOServer.name\n"
    "    VirtualIOServer.nvram\n"
    "    VirtualIOServer.operating_system\n"
    "    VirtualIOServer.partition_uuid\n"
    "    VirtualIOServer.pending_proc_compat_mode\n"
    "    VirtualIOServer.pending_secure_boot\n"
    "    VirtualIOServer.phys_vols\n"
    "    VirtualIOServer.proc_compat_mode\n"
    "    VirtualIOServer.proc_config\n"
    "    VirtualIOServer.profile_sync\n"
    "    VirtualIOServer.ref_code\n"
    "    VirtualIOServer.ref_code_full\n"
    "    VirtualIOServer.rmc_ip\n"
    "    VirtualIOServer.rmc_state\n"
    "    VirtualIOServer.scsi_mappings\n"
    "    VirtualIOServer.seas\n"
    "    VirtualIOServer.state\n"
    "    VirtualIOServer.trunk_adapters\n"
    "    VirtualIOServer.uptime\n"
    "    VirtualIOServer.uuid\n"
    "    VirtualIOServer.vfc_mappings\n"
    "    VirtualIOServer.vnic_capable\n"
    "    VirtualIOServer.vnic_failover_capable\n"
    "    VirtualMediaRepository.name\n"
    "    VirtualMediaRepository.optical_media\n"
    "    VirtualMediaRepository.size\n"
    "    VirtualOpticalMedia.media_name\n"
    "    VirtualOpticalMedia.mount_type\n"
    "    VirtualOpticalMedia.name\n"
    "    VirtualOpticalMedia.size\n"
    "    VirtualOpticalMedia.udid\n"
    "    VirtualOpticalTargetDevice.lua\n"
    "    VirtualOpticalTargetDevice.name\n"
    "    VirtualSCSIMapping.backing_storage\n"
    "    VirtualSCSIMapping.client_adapter\n"
    "    VirtualSCSIMapping.client_lpar_href\n"
    "    VirtualSCSIMapping.server_adapter\n"
    "    VirtualSCSIMapping.target_dev\n"])


class VirtualIOServerTest(twa.TestWrapper):
    ''' Test framework for VirtualIOServer Object Type. '''
    @property
    def file(self):
        return 'fake_vios_mappings.txt'

    @property
    def wrapper_class_to_test(self):
        return vios_w.VIOS

    def test_virtual_io_server(self):
        vios = vios_c.VirtualIOServer()

        field_models = [vios_w.VIOS,
                        bp_w.PartitionCapabilities,
                        bp_w.PartitionProcessorConfiguration,
                        bp_w.PartitionMemoryConfiguration,
                        bp_w.SharedProcessorConfiguration,
                        bp_w.DedicatedProcessorConfiguration,
                        bp_w.PartitionIOConfiguration,
                        bp_w.IOSlot,
                        bp_w.IOSlot.AssociatedIOSlot,
                        bp_w.IOAdapter,
                        bp_w.PhysFCAdapter,
                        bp_w.PhysFCPort,
                        bp_w.TaggedIO,
                        n.SEA,
                        n.TrunkAdapter,
                        n.EthernetBackingDevice,
                        stor.VSCSIClientAdapterElement,
                        stor.VSCSIServerAdapterElement,
                        stor.VFCClientAdapterElement,
                        stor.VFCServerAdapterElement,
                        stor.LU,
                        stor.PV,
                        stor.VMediaRepos,
                        stor.VOptMedia,
                        stor.VDisk,
                        vios_w.VSCSIMapping,
                        vios_w.VFCMapping,
                        vios_w.LinkAggrIOAdapterChoice,
                        stor.LUTargetDev,
                        stor.PVTargetDev,
                        stor.VDiskTargetDev,
                        stor.VOptTargetDev]
        name_to_ops = {'create', 'list', 'update', 'delete',
                       'power-on', 'power-off', 'restart'}

        test_util.test_obj_type(self, vios, 'VirtualIOServer', vios_w.VIOS,
                                field_models, name_to_ops,
                                exp_help=help_vios.help_text)

        # Test Operations
        test_util.test_ops(
            self, parent_model_ops={vios.name_to_ops['list']: None,
                                    vios.name_to_ops['update']: None,
                                    vios.name_to_ops['delete']: None,
                                    vios.name_to_ops['power-on']: None,
                                    vios.name_to_ops['power-off']: None,
                                    vios.name_to_ops['restart']: None,
                                    vios.name_to_ops['create']:
                                    [ms_w.System]},
            child_model_ops={vios.name_to_ops['list']: None,
                             vios.name_to_ops['update']: None,
                             vios.name_to_ops['delete']: None,
                             vios.name_to_ops['power-on']: None,
                             vios.name_to_ops['power-off']: None,
                             vios.name_to_ops['restart']: None,
                             vios.name_to_ops['create']: None},
            elem_wrp_func_ops={vios.name_to_ops['list']: None,
                               vios.name_to_ops['update']: None,
                               vios.name_to_ops['delete']: None,
                               vios.name_to_ops['power-on']: None,
                               vios.name_to_ops['power-off']: None,
                               vios.name_to_ops['restart']: None,
                               vios.name_to_ops['create']: None})

    def test_list(self):
        with open('pvmcli/tests/data/vios_all_out.txt') as vios_ao_file:
            test_util.test_list(
                self, 'VirtualIOServer',
                LIST_USAGE_HELP + test_util.EXPECTED_HELP + DISPLAY_FIELD_HELP,
                ["Virtual I/O Servers\n"
                 "+-------------+----+---------+--------+----------+-------+"
                 "-----+-----+\n"
                 "|     Name    | ID |  State  |  RMC   | Ref Code |  Mem  |"
                 " CPU | Ent |\n"
                 "+-------------+----+---------+--------+----------+-------+"
                 "-----+-----+\n"
                 "| vios_1_Drew | 1  | running | active |          | 28672 |"
                 "  16 | 1.6 |\n"
                 "+-------------+----+---------+--------+----------+"
                 "-------+-----+-----+\n"],
                [vios_ao_file.read()],
                ["<VirtualIOServer:VirtualIOServer xmlns:VirtualIOServer"],
                ['name', 'rmc_ip', 'VirtualIOServer.state'],
                ["name=vios_1_Drew,rmc_ip=9.1.2.4,state=running\n"],
                ["vios_1_Drew,9.1.2.4,running\n"],
                ["name___vios_1_Drew,rmc_ip___9.1.2.4,state___running\n"],
                ["name=vios_1_Drew___rmc_ip=9.1.2.4___state=running\n"],
                ["name=vios_1_Drew,rmc_ip=9.1.2.4,state=running\n"],
                ["name___vios_1_Drew___rmc_ip___9.1.2.4___state___running\n"],
                'VirtualIOServer')

        cp = test_util.setup_mock(self, args=[
            'VirtualIOServer', 'list'], adpt_fx=self.adpt)
        cp.obj_type.ec.wrappers = []
        l_exc = self.assertRaises(exc.NoEntriesFoundError, cp.obj_type.execute)
        self.assertEqual(l_exc.ret_code, 0)

    def test_create(self):
        test_util.setup_mock(self, args=[
            'VirtualIOServer', 'create', '--name', 'vios_name', '--mem',
            '1024', '--proc-type', 'dedicated', '--proc', '1',
            '--sharing-mode', 'normal'], adpt_fx=self.adpt)

        test_util.setup_mock(self, args=[
            'VirtualIOServer', 'create', '--name', 'vios_name', '--mem',
            '1024', '--proc-unit', '1', '--proc-type', 'shared', '--proc', '1',
            '--sharing-mode', 'capped'], adpt_fx=self.adpt)

        # Test adding ProfileIOSlot
        test_util.setup_mock(self, args=[
            'VirtualIOServer', 'create', '--name', 'vios_name', '--mem',
            '1024', '--proc-type', 'dedicated', '--proc', '1',
            '--sharing-mode', 'normal', '--drc-name',
            'io_slot_name'], adpt_fx=self.adpt)

        # Type is inferred for VirtualIOServer creation
        cp = cli_parser.CLIParser(args=['VirtualIOServer', 'create', '--type',
                                        'AIX/Linux'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No name argument
        cp = cli_parser.CLIParser(args=['VirtualIOServer', 'create', '--mem',
                                        '1024', '--proc-type', 'shared',
                                        '--proc', '1', '--sharing-mode',
                                        'capped'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No mem argument
        cp = cli_parser.CLIParser(args=['VirtualIOServer', 'create', '--name',
                                        'vios_name', '--proc-type', 'shared',
                                        '--proc', '1', '--sharing-mode',
                                        'capped'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No proc type argument
        cp = cli_parser.CLIParser(args=['VirtualIOServer', 'create', '--name',
                                        'vios_name', '--mem', '1',
                                        '--proc', '1', '--sharing-mode',
                                        'capped'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No proc argument
        cp = cli_parser.CLIParser(args=['VirtualIOServer', 'create', '--name',
                                        'vios_name', '--proc-type', 'shared',
                                        '--mem', '1024', '--sharing-mode',
                                        'capped'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

        # No sharing-mode argument
        cp = cli_parser.CLIParser(args=['VirtualIOServer', 'create', '--name',
                                        'vios_name', '--proc-type', 'shared',
                                        '--mem', '1024', '--proc', '1'])
        se_exc = self.assertRaises(SystemExit, cp.process)
        self.assertEquals(se_exc.code, exc.ARGPARSE_ERR_RET_CODE)

    def test_update(self):
        # TODO (clbush): Update test
        pass

    @mock.patch("pypowervm.tasks.power.power_on")
    def test_power_on(self, power_on_mock):
        test_util.test_power_on(self, power_on_mock, test_util.VIOS_OBJ_NAME)

    @mock.patch("pypowervm.tasks.power.PowerOp.stop")
    def test_power_off_restart(self, power_off_mock):
        test_util.test_power_off(self, power_off_mock=power_off_mock,
                                 obj_name=test_util.VIOS_OBJ_NAME)

    @mock.patch('pypowervm.wrappers.entry_wrapper.EntryWrapper.search')
    @mock.patch('pypowervm.wrappers.entry_wrapper.EntryWrapper.delete')
    def test_delete(self, ew_mock_delete, search_mock):
        ''' Test delete operation on VirtualIOServer ObjectType. '''
        search_mock.return_value = [self.dwrap]
        test_util.test_delete(self, ew_mock_delete, 'VirtualIOServer',
                              DELETE_USAGE_HELP, wrps=[self.dwrap])

    DEFAULT_ALL_OUT = ("-------------------\n"
                       "| VirtualIOServer |\n"
                       "-------------------\n"
                       "  allow_perf_data_collection=True\n"
                       "  assoc_sys_uuid=726e9cb3-6576-3df5-ab60-40893d51"
                       "d074\n"
                       "  avail_priority=191\n"
                       "  current_secure_boot=0\n"
                       "  disable_secure_boot=False\n"
                       "  env=Virtual IO Server\n"
                       "  id=1\n"
                       "  ip_addresses=('9.1.2.4',)\n"
                       "  is_license_accepted=True\n"
                       "  is_mgmt_partition=False\n"
                       "  is_mover_service_partition=True\n"
                       "  is_service_partition=True\n"
                       "  keylock_pos=normal\n"
                       "  name=vios_1_Drew\n"
                       "  operating_system=VIOS 2.2.3.3 6100-09-03-1415\n"
                       "  partition_uuid=014C1F78-B210-40B2-A12E-C82316B87"
                       "8CA\n"
                       "  pending_proc_compat_mode=default\n"
                       "  pending_secure_boot=0\n"
                       "  proc_compat_mode=POWER7\n"
                       "  profile_sync=True\n"
                       "  rmc_ip=9.1.2.4\n"
                       "  rmc_state=active\n"
                       "  state=running\n"
                       "  uuid=014C1F78-B210-40B2-A12E-C82316B878CA\n")

    # Show is_primary is not in output and tagged_vlan list shows up
    SHARED_ETH_ALL_OUT = (
        "  -------------------------\n"
        "  | SharedEthernetAdapter |\n"
        "  -------------------------\n"
        "    dev_name=ent14\n"
        "    ha_mode=disabled\n"
        "    pvid=1\n"
        "    ----------------\n"
        "    | TrunkAdapter |\n"
        "    ----------------\n"
        "      dev_name=ent11\n"
        "      has_tag_support=True\n"
        "      loc_code=U8246.L2C.0604C7A-V1-C3\n"
        "      pvid=4094\n"
        "      tagged_vlans=[29, 100, 123, 1000, 2227, 2777, "
        "2881, 3001, 3901]\n"
        "      trunk_pri=1\n"
        "      varied_on=True\n"
        "      vios_id=1\n"
        "      virtual_slot_number=3\n"
        "      vswitch_id=0\n"
        "    -------------------------\n")

    def test_where_all_out(self):
        # Test --all-out
        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '--all-out', '--where',
                        'VirtualIOServer.name=abcd'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertEqual('\n', cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '--all-out', '-w',
                        'VirtualIOServer.name!=abcd'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertIn(self.DEFAULT_ALL_OUT, cp.obj_type.cli_out.getvalue())
        self.assertIn(self.SHARED_ETH_ALL_OUT, cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '--all-out', '--where',
                        'VirtualIOServer.name=vios_1_Drew'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertIn(self.DEFAULT_ALL_OUT, cp.obj_type.cli_out.getvalue())
        self.assertIn(self.SHARED_ETH_ALL_OUT, cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '--all-out', '--where',
                        'VirtualIOServer.name=vios_1_Drew',
                        'SharedProcessorConfiguration.min_units   =  0.1'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertIn(self.DEFAULT_ALL_OUT, cp.obj_type.cli_out.getvalue())
        self.assertIn(self.SHARED_ETH_ALL_OUT, cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '--all-out', '--where',
                        'VirtualIOServer.name=vios_1_Drew',
                        'SharedProcessorConfiguration.min_units!=  0.1'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertEqual('\n', cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '--all-out', '--where',
                        'VirtualIOServer.name=vios_1_Drew',
                        'SharedProcessorConfiguration.min_units   =  0.1',
                        'VirtualIOServer.id=1'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertIn(self.DEFAULT_ALL_OUT, cp.obj_type.cli_out.getvalue())
        self.assertIn(self.SHARED_ETH_ALL_OUT, cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '--all-out', '--where',
                        'VirtualIOServer.name=vios_1_Drew',
                        'SharedProcessorConfiguration.min_units   =  0.5',
                        'VirtualIOServer.id!=1'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertEqual('\n', cp.obj_type.cli_out.getvalue())

    def test_where_spec_disp(self):
        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '-d', 'name', '--where',
                        'VirtualIOServer.name=vios_1_Drew'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertEqual('name=vios_1_Drew\n', cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '-d', 'name', '--where',
                        'VirtualIOServer.name=vios_1_Drew',
                        'IOAdapter.dev_name!=notathing'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertEqual('name=vios_1_Drew\n', cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '-d', 'name', '--where',
                        'VirtualIOServer.name=vios_1_Drew',
                        'IOAdapter.dev_name=notathing'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertEqual('\n', cp.obj_type.cli_out.getvalue())

        cp = test_util.setup_mock(
            self, args=['VirtualIOServer', 'list', '-d', 'IOAdapter.dev_name',
                        'ProfileIOSlot.pci_class',
                        'PartitionMemoryConfiguration.min'],
            adpt_fx=self.adpt, wrps=[self.dwrap])
        cp.obj_type.execute()
        self.assertEqual("dev_name=U78AB.001.WZSH5ZY-P1-T5,"
                         "pci_class=1003,min=1024\n"
                         "dev_name=U78AB.001.WZSH5ZY-P1-C18,"
                         "pci_class=FFFF,min=1024\n"
                         "dev_name=U78AB.001.WZSH5ZY-P1-C7,"
                         "pci_class=0200,min=1024\n"
                         "dev_name=U78AB.001.WZSH5ZY-P1-C2,"
                         "pci_class=FFFF,min=1024\n"
                         "dev_name=U78AB.001.WZSH5ZY-P1-C3,"
                         "pci_class=0000,min=1024\n",
                         cp.obj_type.cli_out.getvalue())
