#
# Licensed Materials - Property of IBM
# COPYRIGHT International Business Machines Corp. 2015, 2017
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import mock
import six

import pvmcli.command.operation as op
import pvmcli.command.virtual_scsi_mapping as scsi_c
import pvmcli.help.help_virtual_scsi_mapping as help_scsi
import pvmcli.tests.util.test_util as test_util
import pvmcli.util.exceptions as e
import pypowervm.tests.test_utils.test_wrapper_abc as twa
import pypowervm.wrappers.base_partition as bp
import pypowervm.wrappers.logical_partition as lpar_w
import pypowervm.wrappers.storage as s
import pypowervm.wrappers.virtual_io_server as vios_w
from pypowervm.tests.test_utils import pvmhttp

LIST_USAGE_HELP = (["usage: pvmctl [help | --help] VirtualSCSIMapping list "
                    "{--parent-id | -p} <VirtualIOServer field>=value "
                    "{--object-id | -i} <VirtualSCSIMapping field>=value "
                    "{args}"])

CREATE_USAGE_HELP = ("usage: pvmctl [help | --help] VirtualSCSIMapping create "
                     "{--parent-id | -p} <VirtualIOServer field>=value "
                     "{--object-id | -i} <VirtualSCSIMapping field>="
                     "value {args}")

REMAP_USAGE_HELP = ("usage: pvmctl [help | --help] VirtualSCSIMapping remap "
                    "{--parent-id | -p} <VirtualIOServer field>=value "
                    "{--object-id | -i} <VirtualSCSIMapping field>="
                    "value {args}")

DELETE_USAGE_HELP = ("usage: pvmctl [help | --help] VirtualSCSIMapping delete "
                     "{--parent-id | -p} <VirtualIOServer field>=value "
                     "{--object-id | -i} <VirtualSCSIMapping field>="
                     "value {args}")

OPT_ARGS_HELP = ["optional arguments:\n",
                 "  --vg VG               name=value identifier search"
                 " field to determine the volume group owner of the "
                 "specified storage.\n"
                 "                        Examples: serial_id=1\n"
                 "                                  name=vg_name\n",
                 "  --ssp SSP             name=value identifier search"
                 " field to determine the shared storage pool owner of "
                 "the specified logical unit.\n"
                 "                        Examples: udid=1234abcd\n"
                 "                                  name=ssp_name\n"]

CREATE_REQ_ARGS_HELP = (["required arguments:\n",
                         "  --type {lv,vopt,lu,pv,file}\n"
                         "                        Type of storage being "
                         "added.\n"
                         "                        Options:\n"
                         "                        lv (Logical Volume)\n"
                         "                        vopt (Virtual Optical "
                         "Media)\n"
                         "                        lu (Logical Unit)\n"
                         "                        pv (Physical Volume)\n"
                         "                        file (File I/O)\n",
                         "  --lpar LPAR           name=value identifier "
                         "search field to determine the LPAR for the storage "
                         "connection.\n"
                         "                        Examples: id=1\n"
                         "                                  name=lpar_name\n",
                         "  --stor-id STOR_ID     Limited name=value "
                         "identifier search field to determine the storage "
                         "being added. Valid Identifiers: name, udid, path "
                         "(--type file only). If not specified, default to "
                         "name."])

REMAP_REQ_ARGS_HELP = (["required arguments:\n",
                        "  --type {vopt}\n",
                        "  --stor-id STOR_ID     Limited name=value "
                        "identifier search field to determine the storage "
                        "being modified. Valid Identifiers: name, udid. "
                        "If not specified, default to name.\n",
                        "  --new-stor-id NEW_STOR_ID\n"
                        "                        Identifier to determine "
                        "storage element replacing existing one. "
                        "Valid Identifiers: name, udid. "
                        "If not specified, default to name.\n",
                        "  --lpar LPAR           name=value identifier "
                        "search field to determine the LPAR for the storage "
                        "connection.\n"
                        "                        Examples: id=1\n"
                        "                                  name=lpar_name"])

DELETE_REQ_ARGS_HELP = (["  --type {lv,vopt,lu,pv,file}\n"
                         "                        Type of storage being "
                         "removed.\n"
                         "                        Options:\n"
                         "                        lv (Logical Volume)\n"
                         "                        vopt (Virtual Optical "
                         "Media)\n"
                         "                        lu (Logical Unit)\n"
                         "                        pv (Physical Volume)\n",
                         "                        file (File I/O)\n",
                         "  --lpar LPAR           name=value identifier "
                         "search field to determine the LPAR for the storage "
                         "connection.\n"
                         "                        Examples: id=1\n"
                         "                                  name=lpar_name\n",
                         "  --stor-id STOR_ID     Limited name=value "
                         "identifier search field to determine the storage "
                         "being removed. Valid Identifiers: name, udid, path "
                         "(--type file only). If not specified, default to "
                         "name."])

DISPLAY_FIELD_HELP = (
    ["    ClientAdapter.is_varied_on\n"
     "    ClientAdapter.loc_code\n"
     "    ClientAdapter.lpar_id\n"
     "    ClientAdapter.lpar_slot_num\n"
     "    ClientAdapter.side\n"
     "    ClientAdapter.vios_id\n"
     "    ClientAdapter.vios_slot_num\n"
     "    LogicalUnit.capacity\n"
     "    LogicalUnit.cloned_from_udid\n"
     "    LogicalUnit.emulate_model\n"
     "    LogicalUnit.in_use\n"
     "    LogicalUnit.is_thin\n"
     "    LogicalUnit.lu_type\n"
     "    LogicalUnit.name\n"
     "    LogicalUnit.tag\n"
     "    LogicalUnit.udid\n"
     "    LogicalVolumeVirtualTargetDevice.lua\n"
     "    LogicalVolumeVirtualTargetDevice.name\n"
     "    PhysicalVolume.avail_for_use\n"
     "    PhysicalVolume.capacity\n"
     "    PhysicalVolume.description\n"
     "    PhysicalVolume.emulate_model\n"
     "    PhysicalVolume.is_fc_backed\n"
     "    PhysicalVolume.loc_code\n"
     "    PhysicalVolume.name\n"
     "    PhysicalVolume.read_iops_limit\n"
     "    PhysicalVolume.state\n"
     "    PhysicalVolume.tag\n"
     "    PhysicalVolume.udid\n"
     "    PhysicalVolume.write_iops_limit\n"
     "    PhysicalVolumeVirtualTargetDevice.lua\n"
     "    PhysicalVolumeVirtualTargetDevice.name\n"
     "    ServerAdapter.backing_dev_name\n"
     "    ServerAdapter.is_varied_on\n"
     "    ServerAdapter.loc_code\n"
     "    ServerAdapter.lpar_id\n"
     "    ServerAdapter.lpar_slot_num\n"
     "    ServerAdapter.name\n"
     "    ServerAdapter.side\n"
     "    ServerAdapter.udid\n"
     "    ServerAdapter.vios_id\n"
     "    ServerAdapter.vios_slot_num\n"
     "    SharedStoragePoolLogicalUnitVirtualTargetDevice.lua\n"
     "    SharedStoragePoolLogicalUnitVirtualTargetDevice.name\n"
     "    VirtualDisk.backstore_type\n"
     "    VirtualDisk.capacity\n"
     "    VirtualDisk.emulate_model\n"
     "    VirtualDisk.file_format\n"
     "    VirtualDisk.label\n"
     "    VirtualDisk.name\n"
     "    VirtualDisk.read_iops_limit\n"
     "    VirtualDisk.tag\n"
     "    VirtualDisk.udid\n"
     "    VirtualDisk.vdtype\n"
     "    VirtualDisk.vg_uri\n"
     "    VirtualDisk.write_iops_limit\n"
     "    VirtualOpticalMedia.media_name\n"
     "    VirtualOpticalMedia.mount_type\n"
     "    VirtualOpticalMedia.name\n"
     "    VirtualOpticalMedia.size\n"
     "    VirtualOpticalMedia.udid\n"
     "    VirtualOpticalTargetDevice.lua\n"
     "    VirtualOpticalTargetDevice.name\n"
     "    VirtualSCSIMapping.backing_storage\n"
     "    VirtualSCSIMapping.client_adapter\n"
     "    VirtualSCSIMapping.client_lpar_href\n"
     "    VirtualSCSIMapping.server_adapter\n"
     "    VirtualSCSIMapping.target_dev\n"])

bs_str = 'pypowervm.wrappers.virtual_io_server.VSCSIMapping.backing_storage'
repo_str = 'pypowervm.wrappers.storage.VG.vmedia_repos'


class SCSIMappingTest(twa.TestWrapper):
    ''' Test framework for SCSIMapping Object Type. '''
    @property
    def file(self):
        return 'fake_vios_mappings.txt'

    @property
    def wrapper_class_to_test(self):
        return vios_w.VSCSIMapping

    def test_scsi_mapping(self):
        scsi = scsi_c.SCSIStorageMapping()

        test_util.test_obj_type(
            self, scsi, 'VirtualSCSIMapping', vios_w.VSCSIMapping,
            field_models=[vios_w.VSCSIMapping, s.VDisk, s.VOptMedia, s.LU,
                          s.PV, s.VSCSIClientAdapterElement,
                          s.VSCSIServerAdapterElement, s.LUTargetDev,
                          s.PVTargetDev, s.VDiskTargetDev, s.VOptTargetDev],
            name_to_ops={op.CREATE_NAME, op.LIST_NAME,
                         op.DELETE_NAME, op.REMAP_NAME},
            exp_help=help_scsi.help_op)

        # Test Operations
        test_util.test_ops(
            self, parent_model_ops={scsi.name_to_ops['list']: [vios_w.VIOS],
                                    scsi.name_to_ops['create']: [vios_w.VIOS],
                                    scsi.name_to_ops['remap']: [vios_w.VIOS],
                                    scsi.name_to_ops['delete']: [vios_w.VIOS]},
            child_model_ops={scsi.name_to_ops['list']: None,
                             scsi.name_to_ops['create']: None,
                             scsi.name_to_ops['remap']: None,
                             scsi.name_to_ops['delete']: None},
            elem_wrp_func_ops={scsi.name_to_ops['list']:
                               [vios_w.VIOS.scsi_mappings],
                               scsi.name_to_ops['create']:
                               [vios_w.VIOS.scsi_mappings],
                               scsi.name_to_ops['remap']:
                               [vios_w.VIOS.scsi_mappings],
                               scsi.name_to_ops['delete']:
                               [vios_w.VIOS.scsi_mappings]})

    @mock.patch("pypowervm.wrappers.virtual_io_server.VIOS.get")
    @mock.patch("pypowervm.wrappers.logical_partition.LPAR.get")
    def test_list(self, lpar_mock, vios_mock):
        test_lp_wrp = lpar_w.LPAR.bld(
            mock.Mock(), "test_lpar",
            bp.PartitionMemoryConfiguration.bld(mock.Mock(), 1024),
            bp.PartitionProcessorConfiguration.bld_dedicated(mock.Mock(), 1))
        test_lp_wrp.set_parm_value('PartitionID', 15)
        lpar_mock.return_value = [test_lp_wrp]
        vios_sc_wrap = vios_w.VIOS.wrap(self.resp)
        vios_sc_wrap.set_parm_value('PartitionID', 1)
        vios_mock.return_value = [vios_sc_wrap]
        self.assertEqual(vios_mock()[0].id, 1)
        self.assertEqual(vios_sc_wrap.scsi_mappings[12].server_adapter.vios_id,
                         1)
        self.assertEqual(vios_sc_wrap.scsi_mappings[0].server_adapter.vios_id,
                         1)
        test_util.test_list(
            self, 'VirtualSCSIMapping',
            LIST_USAGE_HELP + test_util.EXPECTED_HELP + DISPLAY_FIELD_HELP,
            ["Virtual SCSI Mappings\n"
             "+-----------+-----------+-------------+-----------+"
             "--------------+\n"
             "|    LPAR   | LPAR Slot |     VIOS    | VIOS Slot |"
             "   Storage    |\n"
             "+-----------+-----------+-------------+-----------+"
             "--------------+\n"
             "|   <None>  |     2     | vios_1_Drew |     46    |"
             "              |\n"
             "| test_lpar |     2     | vios_1_Drew |     25    |"
             " asdfasfdasdf |\n"
             "+-----------+-----------+-------------+-----------+"
             "--------------+\n"],
            ["----------------------\n"
             "| VirtualSCSIMapping |\n"
             "----------------------\n"
             "  client_lpar_href=https://9.1.2.3:12443/rest/api/uom/ManagedSy"
             "stem/726e9cb3-6576-3df5-ab60-40893d51d074/LogicalPartition/0C0A"
             "6EBE-7BF4-4707-8780-A140F349E42E\n"
             "  ---------------\n"
             "  | VirtualDisk |\n"
             "  ---------------\n"
             "    capacity=0.125\n"
             "    emulate_model=True\n"
             "    label=None\n"
             "    name=asdfasfdasdf\n"
             "    udid=0300004c7a00007a00000001466c54110f.16\n"
             "  -----------------\n"
             "  | ClientAdapter |\n"
             "  -----------------\n"
             "    is_varied_on=true\n"
             "    loc_code=U8246.L2C.0604C7A-V15-C2\n"
             "    lpar_id=15\n"
             "    lpar_slot_num=2\n"
             "    side=Client\n"
             "    vios_id=1\n"
             "    vios_slot_num=25\n"
             "  -----------------\n"
             "  | ServerAdapter |\n"
             "  -----------------\n"
             "    backing_dev_name=asdfasfdasdf\n"
             "    is_varied_on=true\n"
             "    loc_code=U8246.L2C.0604C7A-V1-C25\n"
             "    lpar_id=15\n"
             "    lpar_slot_num=2\n"
             "    name=vhost16\n"
             "    side=Server\n"
             "    udid=1eU8246.L2C.0604C7A-V1-C25\n"
             "    vios_id=1\n"
             "    vios_slot_num=25\n"],
            ['<VirtualSCSIMapping xmlns="http://www.ibm.com/xmlns/'],
            ['ServerAdapter.lpar_slot_num',
             'ClientAdapter.loc_code',
             'ServerAdapter.loc_code'],
            ["lpar_slot_num=2,loc_code=U8246.L2C.0604C7A-V15-C2,"
             "loc_code=U8246.L2C.0604C7A-V1-C25\nlpar_slot_num=2,"
             "loc_code=,loc_code=U8246.L2C.0604C7A-V1-C46\n"],
            ["2,U8246.L2C.0604C7A-V15-C2,U8246.L2C.0604C7A-V1-C25\n"],
            ["lpar_slot_num___2,loc_code___U8246.L2C.0604C7A-V15-C2,"
             "loc_code___U8246.L2C.0604C7A-V1-C25\n"],
            ["lpar_slot_num=2___loc_code=U8246.L2C.0604C7A-V15-C2___"
             "loc_code=U8246.L2C.0604C7A-V1-C25\n"],
            ["lpar_slot_num=2,loc_code=U8246.L2C.0604C7A-V15-C2,"
             "loc_code=U8246.L2C.0604C7A-V1-C25___lpar_slot_num=2,loc_code=,"
             "loc_code=U8246.L2C.0604C7A-V1-C46\n"],
            ["lpar_slot_num___2___loc_code___U8246.L2C.0604C7A-V15-C2"
             "___loc_code___U8246.L2C.0604C7A-V1-C25___lpar_slot_num___2"
             "___loc_code______loc_code___U8246.L2C.0604C7A-V1-C46\n"],
            'VirtualIOServer',
            wrps=[vios_sc_wrap.scsi_mappings[12],
                  vios_sc_wrap.scsi_mappings[0]])

    def _setup_mock_vg(self):
        resp = pvmhttp.PVMResp(
            pvmfile=pvmhttp.PVMFile('fake_volume_group.txt'),
            adapter=self.adpt).get_response()
        return s.VG.wrap(resp)

    def _setup_mock_ssp(self):
        resp = pvmhttp.PVMResp(
            pvmfile=pvmhttp.PVMFile('ssp.txt'),
            adapter=self.adpt).get_response()
        return s.SSP.wrap(resp)

    def _mock_p_wrps(self, cp, vios_wrap):
        cp.obj_type.ec.elem_p_wrappers = [vios_wrap]

    @mock.patch('pypowervm.wrappers.storage.FileIO.bld')
    @mock.patch('pypowervm.wrappers.logical_partition.LPAR.search')
    @mock.patch('pypowervm.wrappers.storage.VG.search')
    @mock.patch('pypowervm.tasks.scsi_mapper.add_vscsi_mapping')
    def test_create(self, add_vscsi_mock, vg_search_mock, lpar_search_mock,
                    fileio_mock):
        vios_wrap = vios_w.VIOS.wrap(self.resp)
        vg_wrap = self._setup_mock_vg()
        file_io_ret = mock.Mock()
        fileio_mock.return_value = file_io_ret

        # pvmctl <obj_name> create help
        help_out = six.StringIO()
        test_util.setup_and_exec(
            self, ['VirtualSCSIMapping', 'create', 'help'], help_out=help_out)
        self.assertEqual(str(help_out.getvalue()),
                         str(help_scsi.help_create) + "\n")
        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        s.VG.wrap = mock.Mock(return_value=vg_wrap)

        # pvmctl scsi create --lpar lpar_name --type file \
        # --stor-id path=testing
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'file', '--stor-id', 'path=testing']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        fileio_mock.assert_called_once_with(cp.obj_type.ec.sm.adapter,
                                            'testing')
        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d', vios_wrap,
            lpar_search_mock().__getitem__().uuid, file_io_ret)
        add_vscsi_mock.reset_mock()
        fileio_mock.reset_mock()

        # pvmctl scsi create --lpar lpar_name --type file \
        # --stor-id path=testing --bstype user:qcow
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'file', '--stor-id', 'path=testing', '--bstype', 'user:qcow']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        fileio_mock.assert_called_once_with(cp.obj_type.ec.sm.adapter,
                                            'testing',
                                            backstore_type='user:qcow')
        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d', vios_wrap,
            lpar_search_mock().__getitem__().uuid, file_io_ret)
        add_vscsi_mock.reset_mock()
        fileio_mock.reset_mock()

        # pvmctl scsi create --lpar lpar_name --type file \
        # --stor-id path=testing --bstype fileio
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'file', '--stor-id', 'path=testing', '--bstype', 'fileio']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        fileio_mock.assert_called_once_with(cp.obj_type.ec.sm.adapter,
                                            'testing', backstore_type='fileio')
        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d', vios_wrap,
            lpar_search_mock().__getitem__().uuid, file_io_ret)
        add_vscsi_mock.reset_mock()
        fileio_mock.reset_mock()

        # pvmctl scsi create --lpar lpar_name --type file \
        # --stor-id path=testing --bstype uSeR:qCOw
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'file', '--stor-id', 'path=testing', '--bstype', 'uSeR:qCOw']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        fileio_mock.assert_called_once_with(cp.obj_type.ec.sm.adapter,
                                            'testing',
                                            backstore_type='user:qcow')
        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d', vios_wrap,
            lpar_search_mock().__getitem__().uuid, file_io_ret)
        add_vscsi_mock.reset_mock()
        fileio_mock.reset_mock()

        # pvmctl scsi create --lpar lpar_name --type file \
        # --stor-id path=testing --bstype FILeIo
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'file', '--stor-id', 'path=testing', '--bstype', 'FILeIo']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        fileio_mock.assert_called_once_with(cp.obj_type.ec.sm.adapter,
                                            'testing', backstore_type='fileio')
        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d', vios_wrap,
            lpar_search_mock().__getitem__().uuid, file_io_ret)
        add_vscsi_mock.reset_mock()
        fileio_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type vopt \
        # --stor-id name=blank_media_2
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'vopt', '--stor-id', 'name=blank_media_2']
        cp = test_util.setup_mock(
            self, args, self.adpt,
            wrps=[vg_wrap.vmedia_repos[0].optical_media[1]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d', vios_wrap,
            lpar_search_mock().__getitem__().uuid,
            vg_wrap.vmedia_repos[0].optical_media[1])
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type lv \
        # --stor-id udid=0300f8d6de00004b000000014a54555cd9.1
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'lv', '--stor-id',
                ("udid=%s" % vg_wrap.virtual_disks[0].udid)]
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.virtual_disks[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d',
            vios_wrap, lpar_search_mock().__getitem__().uuid,
            vg_wrap.virtual_disks[0])
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type pv \
        # --stor-id hdisk1
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk1']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d',
            vios_wrap, lpar_search_mock().__getitem__().uuid,
            vios_wrap.phys_vols[2])
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type pv \
        # --stor-id hdisk1 --vg name=image_pool
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk1', '--vg', 'name=image_pool']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        vg_search_mock.return_value = vg_wrap

        cp.obj_type.execute()

        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d',
            vios_wrap, lpar_search_mock().__getitem__().uuid,
            vios_wrap.phys_vols[2])
        add_vscsi_mock.reset_mock()

        # Test when search results return a list
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk1', '--vg', 'name=image_pool']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        vg_search_mock.return_value = [vg_wrap]

        cp.obj_type.execute()

        add_vscsi_mock.assert_called_once_with(
            'caae9209-25e5-35cd-a71a-ed55c03f294d',
            vios_wrap, lpar_search_mock().__getitem__().uuid,
            vios_wrap.phys_vols[2])
        add_vscsi_mock.reset_mock()

        # Invalid args

        # pvmctl <obj_name> create --lpar lpar_name --type vopt \
        # --stor-id name=blank_media_3
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'vopt', '--stor-id', 'name=blank_media_3']
        cp = test_util.setup_mock(
            self, args, self.adpt,
            wrps=[vg_wrap.vmedia_repos[0].optical_media[1]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type lv \
        # --stor-id udid=abcd
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'lv', '--stor-id', "udid=abcd"]
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.virtual_disks[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type pv \
        # --stor-id hdisk4
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk4']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type pv \
        # --stor-id hdisk4 --vg name=image_pool
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk4', '--vg', 'name=image_pool']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        vg_search_mock.return_value = vg_wrap

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type pv \
        # --stor-id hdisk2 --vg name=image_pool
        vios_wrap.phys_vols[1].name = vios_wrap.phys_vols[0].name
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk2', '--vg', 'name=image_pool']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vios_wrap.phys_vols[0],
                                        vios_wrap.phys_vols[1]],
                                  p_wrps=[vios_wrap])

        self.assertRaises(e.MultipleFoundDisksError, cp.obj_type.execute)

        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --stor-id storid
        args = ['scsi', 'create', '--lpar', 'lpar_name', '--stor-id', 'storid']

        self.assertRaises(SystemExit, test_util.setup_mock, self, args,
                          self.adpt,
                          wrps=[vg_wrap.vmedia_repos[0].optical_media[1]])

        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --lpar lpar_name --type vopt
        args = ['scsi', 'create', '--lpar', 'lpar_name', '--type', 'vopt']

        self.assertRaises(SystemExit, test_util.setup_mock, self, args,
                          self.adpt,
                          wrps=[vg_wrap.vmedia_repos[0].optical_media[1]])

        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        # pvmctl <obj_name> create --stor-id storid --type vopt
        args = ['scsi', 'create', '--stor-id', 'storid', '--type', 'vopt']

        self.assertRaises(SystemExit, test_util.setup_mock, self, args,
                          self.adpt,
                          wrps=[vg_wrap.vmedia_repos[0].optical_media[1]])

        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

        # pvmctl scsi create --lpar lpar_name --type file \
        # --stor-id path=testing --bstype invalid
        args = ['scsi', 'create', '--lpar', 'name=lpar_name', '--type',
                'file', '--stor-id', 'path=testing', '--bstype', 'invalid']

        self.assertRaises(SystemExit, test_util.setup_mock, self, args,
                          self.adpt)
        add_vscsi_mock.assert_not_called()
        add_vscsi_mock.reset_mock()

    @mock.patch('pypowervm.wrappers.logical_partition.LPAR.search')
    @mock.patch('pypowervm.tasks.scsi_mapper.modify_vopt_mapping')
    def test_remap(self, mod_vopt_mock, lpar_search_mock):
        vios_wrap = vios_w.VIOS.wrap(self.resp)
        mod_vopt_mock.return_value = vios_wrap, mock.Mock()
        vg_wrap = self._setup_mock_vg()

        # pvmctl scsi remap help
        help_out = six.StringIO()
        test_util.setup_and_exec(
            self, ['VirtualSCSIMapping', 'remap', 'help'], help_out=help_out)
        self.assertIn(REMAP_USAGE_HELP, help_out.getvalue())
        for help_txt in REMAP_REQ_ARGS_HELP:
            self.assertIn(help_txt, help_out.getvalue())
        mod_vopt_mock.assert_not_called()
        mod_vopt_mock.reset_mock()

        s.VG.wrap = mock.Mock(return_value=vg_wrap)

        # Use storage element names
        # pvmctl scsi remap --lpar name=lpar_name --type vopt --stor-id \
        # name=blank_media1 --new-stor-id name=blank_media_2
        args = ['scsi', 'remap', '--lpar', 'name=lpar_name', '--type',
                'vopt', '--stor-id', 'name=blank_media1', '--new-stor-id',
                'name=blank_media_2']
        cp = test_util.setup_mock(
            self, args, self.adpt,
            wrps=[vg_wrap.vmedia_repos[0].optical_media[0],
                  vg_wrap.vmedia_repos[0].optical_media[1]])
        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        mod_vopt_mock.assert_called_once_with(
            cp.obj_type.ec.sm.adapter, vios_wrap,
            lpar_search_mock().__getitem__().uuid,
            new_media=vg_wrap.vmedia_repos[0].optical_media[1],
            media_name=vg_wrap.vmedia_repos[0].optical_media[0].name)
        mod_vopt_mock.reset_mock()

        # modify_vopt_mapping bails and returns nothing
        mod_vopt_mock.return_value = vios_wrap, None

        self.assertRaises(e.MappingNoEntriesModifiedError, cp.obj_type.execute)

        mod_vopt_mock.assert_not_called()
        mod_vopt_mock.reset_mock()
        mod_vopt_mock.return_value = vios_wrap, mock.Mock()

        # Use storage element UDIDs
        # pvmctl scsi remap --lpar name=lpar_name --type vopt --stor-id \
        # udid=0eblank_media1 --new-stor-id udid=0eblank_media_2
        args = ['scsi', 'remap', '--lpar', 'name=lpar_name', '--type',
                'vopt', '--stor-id', 'udid=0eblank_media1', '--new-stor-id',
                'udid=0eblank_media_2']
        cp = test_util.setup_mock(
            self, args, self.adpt,
            wrps=[vg_wrap.vmedia_repos[0].optical_media[0],
                  vg_wrap.vmedia_repos[0].optical_media[1]])
        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        mod_vopt_mock.assert_called_once_with(
            cp.obj_type.ec.sm.adapter, vios_wrap,
            lpar_search_mock().__getitem__().uuid,
            new_media=vg_wrap.vmedia_repos[0].optical_media[1],
            media_name=vg_wrap.vmedia_repos[0].optical_media[0].name)
        mod_vopt_mock.reset_mock()

        # Invalid args

        # Nonexistent name
        # pvmctl scsi remap --lpar name=lpar_name --type vopt --stor-id \
        # name=fake_media --new-stor-id name=blank_media_2
        args = ['scsi', 'remap', '--lpar', 'name=lpar_name', '--type',
                'vopt', '--stor-id', 'name=fake_media', '--new-stor-id',
                'name=blank_media_2']
        cp = test_util.setup_mock(
            self, args, self.adpt,
            wrps=[vg_wrap.vmedia_repos[0].optical_media[0],
                  vg_wrap.vmedia_repos[0].optical_media[1]])
        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        mod_vopt_mock.assert_not_called()
        mod_vopt_mock.reset_mock()

        # Nonexistent UDID
        # pvmctl scsi remap --lpar name=lpar_name --type vopt --stor-id \
        # name=blank_media1 --new-stor-id udid=12345
        args = ['scsi', 'remap', '--lpar', 'name=lpar_name', '--type',
                'vopt', '--stor-id', 'name=blank_media1', '--new-stor-id',
                'udid=12345']
        cp = test_util.setup_mock(
            self, args, self.adpt,
            wrps=[vg_wrap.vmedia_repos[0].optical_media[0],
                  vg_wrap.vmedia_repos[0].optical_media[1]])
        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        mod_vopt_mock.assert_not_called()
        mod_vopt_mock.reset_mock()

    @mock.patch('pypowervm.wrappers.virtual_io_server.VIOS.update')
    @mock.patch('pypowervm.wrappers.logical_partition.LPAR.search')
    def test_delete_no_storage(self, lpar_search_mock, vios_upd_mock):
        vios_wrap = vios_w.VIOS.wrap(self.resp)
        lpar_wrap = test_util.mock_lpar(self.adpt)
        lpar_search_mock.return_value = [lpar_wrap]
        # pvmctl scsi delete --udid 1eU8246.L2C.0604C7A-V1-C25
        args = ['scsi', 'delete', '--udid', '1eU8246.L2C.0604C7A-V1-C25']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        scsi_map = cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings[12]
        other_map = cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings[13]
        self.assertEqual(scsi_map.server_adapter.udid,
                         '1eU8246.L2C.0604C7A-V1-C25')
        self.assertEqual(other_map.server_adapter.udid,
                         '1eU8246.L2C.0604C7A-V1-C70')
        self.assertEqual(
            len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings), 28)
        cp.obj_type.execute()
        self.assertEqual(
            len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings), 27)

        vios_upd_mock.assert_called_once_with()
        lpar_search_mock.assert_not_called()
        vios_upd_mock.reset_mock()
        lpar_search_mock.reset_mock()

        # Test UDID overriding other args (filtering is AND only)
        # pvmctl scsi delete --udid 1eU8246.L2C.0604C7A-V1-C25
        # --vios-slot 73
        args = ['scsi', 'delete', '--udid', '1eU8246.L2C.0604C7A-V1-C70',
                '--vios-slot', '73']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_w.VIOS.wrap(self.resp))
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        # One has been removed from the previous update
        self.assertEqual(
            len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings), 27)
        scsi_map = cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings[12]
        other_map = cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings[11]
        self.assertEqual(scsi_map.server_adapter.udid,
                         '1eU8246.L2C.0604C7A-V1-C70')
        self.assertEqual(other_map.server_adapter.lpar_slot_num, 2)
        self.assertEqual(other_map.server_adapter.vios_slot_num, 73)
        cp.obj_type.execute()
        self.assertEqual(
            len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings), 26)
        other_map = cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings[11]
        self.assertEqual(other_map.server_adapter.lpar_slot_num, 2)
        self.assertEqual(other_map.server_adapter.vios_slot_num, 73)

        vios_upd_mock.assert_called_once_with()
        lpar_search_mock.assert_not_called()
        vios_upd_mock.reset_mock()
        lpar_search_mock.reset_mock()

        # Test other args (filtering is AND only)
        # pvmctl scsi delete --vios-slot 46
        args = ['scsi', 'delete', '--vios-slot', '46']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_w.VIOS.wrap(self.resp))
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        # One has been removed from the previous update
        self.assertEqual(
            len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings), 26)
        scsi_map = cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings[0]
        other_map = cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings[1]
        self.assertEqual(scsi_map.server_adapter.udid,
                         '1eU8246.L2C.0604C7A-V1-C46')
        self.assertEqual(scsi_map.server_adapter.lpar_slot_num, 2)
        self.assertEqual(scsi_map.server_adapter.vios_slot_num, 46)
        self.assertEqual(other_map.server_adapter.lpar_slot_num, 2)
        self.assertEqual(other_map.server_adapter.vios_slot_num, 42)
        cp.obj_type.execute()
        self.assertEqual(
            len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings), 25)
        other_map = cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings[0]
        self.assertEqual(other_map.server_adapter.lpar_slot_num, 2)
        self.assertEqual(other_map.server_adapter.vios_slot_num, 42)

        vios_upd_mock.assert_called_once_with()
        lpar_search_mock.assert_not_called()
        vios_upd_mock.reset_mock()
        lpar_search_mock.reset_mock()

        # Test no deletion exception
        args = ['scsi', 'delete', '--udid', 'imnotanudid']
        cp = test_util.setup_mock(self, args, self.adpt)

        self._mock_p_wrps(cp, vios_w.VIOS.wrap(self.resp))
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertEqual(
            len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings), 25)
        self.assertRaises(e.MappingNoEntriesDeletedError, cp.obj_type.execute)
        self.assertEqual(
            len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings), 25)

        vios_upd_mock.assert_not_called()
        lpar_search_mock.assert_not_called()
        vios_upd_mock.reset_mock()
        lpar_search_mock.reset_mock()

        with mock.patch(bs_str, new_callable=mock.PropertyMock) as mock_bs,\
                mock.patch(repo_str) as repo_mock:
            mock_bs.return_value = None
            repo_mock.return_value = []
            vopt_name = 'vopt_de86c46e07004993b412c948bd5047c2'

            # pvmctl scsi delete --type vopt --stor-id \
            # name=vopt_84d7bfcf44964f398e60254776b94d41
            args = ['scsi', 'delete', '--type', 'vopt', '--stor-id',
                    ('name=%s' % vopt_name), '--lpar', 'id=10']
            cp = test_util.setup_mock(self, args, self.adpt)

            self._mock_p_wrps(cp, vios_wrap)
            cp.obj_type.ec._add_parent_wrappers = mock.Mock(
                side_effect=self._mock_p_wrps(cp, vios_wrap))

            scsi_map = vios_wrap.scsi_mappings[23]
            self.assertIsNone(scsi_map.backing_storage)
            self.assertEqual(
                vopt_name, scsi_map.server_adapter.backing_dev_name)

            self.assertEqual(
                25, len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings))
            cp.obj_type.execute()
            self.assertEqual(
                24, len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings))

            vios_upd_mock.assert_called_once_with()
            vios_upd_mock.reset_mock()

            vopt_name = 'bad_name'
            # Invalid arg
            # pvmctl scsi delete --type vopt --stor-id \
            # name=bad_name
            args = ['scsi', 'delete', '--type', 'vopt', '--stor-id',
                    ('name=%s' % vopt_name), '--lpar', 'id=10']
            cp = test_util.setup_mock(self, args, self.adpt)

            self._mock_p_wrps(cp, vios_wrap)
            cp.obj_type.ec._add_parent_wrappers = mock.Mock(
                side_effect=self._mock_p_wrps(cp, vios_wrap))

            self.assertEqual(
                24, len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings))
            self.assertRaises(
                e.NoFoundDisksError, cp.obj_type.execute)
            self.assertEqual(
                24, len(cp.obj_type.ec.elem_p_wrappers[0].scsi_mappings))

            vios_upd_mock.assert_not_called()

    @mock.patch('pypowervm.wrappers.logical_partition.LPAR.search')
    @mock.patch('pypowervm.wrappers.storage.VG.search')
    @mock.patch('pypowervm.tasks.scsi_mapper.remove_vopt_mapping')
    @mock.patch('pypowervm.tasks.scsi_mapper.remove_vdisk_mapping')
    @mock.patch('pypowervm.tasks.scsi_mapper.remove_lu_mapping')
    @mock.patch('pypowervm.tasks.scsi_mapper.remove_pv_mapping')
    def test_delete_storage(self, rem_pv_mock, rem_lu_mock, rem_lv_mock,
                            rem_vopt_mock, vg_search_mock, lpar_search_mock):
        vios_wrap = vios_w.VIOS.wrap(self.resp)
        rem_pv_mock.return_value = vios_wrap, [mock.Mock()]
        rem_lu_mock.return_value = vios_wrap, [mock.Mock()]
        rem_lv_mock.return_value = vios_wrap, [mock.Mock()]
        rem_vopt_mock.return_value = vios_wrap, [mock.Mock()]
        vg_wrap = self._setup_mock_vg()

        # pvmctl <obj_name> delete help
        help_out = six.StringIO()
        test_util.setup_and_exec(
            self, ['VirtualSCSIMapping', 'delete', 'help'], help_out=help_out)
        self.assertEqual(str(help_out.getvalue()),
                         str(help_scsi.help_delete) + "\n")
        rem_pv_mock.assert_not_called()
        rem_pv_mock.reset_mock()
        rem_lv_mock.assert_not_called()
        rem_lv_mock.reset_mock()
        rem_lu_mock.assert_not_called()
        rem_lu_mock.reset_mock()
        rem_vopt_mock.assert_not_called()
        rem_vopt_mock.reset_mock()

        s.VG.wrap = mock.Mock(return_value=vg_wrap)

        # pvmctl <obj_name> delete --lpar lpar_name --type vopt \
        # --stor-id name=blank_media_2
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'vopt', '--stor-id', 'name=blank_media_2']
        cp = test_util.setup_mock(
            self, args, self.adpt,
            wrps=[vg_wrap.vmedia_repos[0].optical_media[1]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        rem_vopt_mock.assert_called_once_with(
            cp.obj_type.ec.sm.adapter, vios_wrap,
            lpar_search_mock().__getitem__().uuid,
            media_name=vg_wrap.vmedia_repos[0].optical_media[1].name)
        rem_vopt_mock.reset_mock()

        # pvmctl <obj_name> delete --lpar lpar_name --type lv \
        # --stor-id udid=0300f8d6de00004b000000014a54555cd9.1
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'lv', '--stor-id',
                ("udid=%s" % vg_wrap.virtual_disks[0].udid)]
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.virtual_disks[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        rem_lv_mock.assert_called_once_with(
            cp.obj_type.ec.sm.adapter,
            vios_wrap, lpar_search_mock().__getitem__().uuid,
            disk_names=[vg_wrap.virtual_disks[0].name])
        rem_lv_mock.reset_mock()

        # pvmctl <obj_name> delete --lpar lpar_name --type pv \
        # --stor-id hdisk1
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk1']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        cp.obj_type.execute()

        rem_pv_mock.assert_called_once_with(
            cp.obj_type.ec.sm.adapter,
            vios_wrap, lpar_search_mock().__getitem__().uuid,
            vg_wrap.phys_vols[0].name)
        rem_pv_mock.reset_mock()

        # pvmctl <obj_name> delete --lpar lpar_name --type pv \
        # --stor-id hdisk1 --vg name=image_pool
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk1', '--vg', 'name=image_pool']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        vg_search_mock.return_value = vg_wrap

        cp.obj_type.execute()

        rem_pv_mock.assert_called_once_with(
            cp.obj_type.ec.sm.adapter,
            vios_wrap, lpar_search_mock().__getitem__().uuid,
            vg_wrap.phys_vols[0].name)
        rem_pv_mock.reset_mock()

        # Invalid args

        # pvmctl <obj_name> delete --lpar lpar_name --type pv \
        # --vg name=image_pool
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'pv', '--vg', 'name=image_pool']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.SCSITypeDeleteRequiresStorIdError,
                          cp.obj_type.execute)

        rem_pv_mock.assert_not_called()
        rem_pv_mock.reset_mock()
        rem_lv_mock.assert_not_called()
        rem_lv_mock.reset_mock()
        rem_lu_mock.assert_not_called()
        rem_lu_mock.reset_mock()
        rem_vopt_mock.assert_not_called()
        rem_vopt_mock.reset_mock()

        # pvmctl <obj_name> delete --lpar lpar_name --type file \
        # --vg name=image_pool
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'file', '--vg', 'name=image_pool']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.SCSITypeDeleteRequiresStorIdError,
                          cp.obj_type.execute)

        rem_pv_mock.assert_not_called()
        rem_pv_mock.reset_mock()
        rem_lv_mock.assert_not_called()
        rem_lv_mock.reset_mock()
        rem_lu_mock.assert_not_called()
        rem_lu_mock.reset_mock()
        rem_vopt_mock.assert_not_called()
        rem_vopt_mock.reset_mock()

        # pvmctl <obj_name> delete --lpar lpar_name --type vopt \
        # --stor-id name=blank_media_3
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'vopt', '--stor-id', 'name=blank_media_3']
        cp = test_util.setup_mock(
            self, args, self.adpt,
            wrps=[vg_wrap.vmedia_repos[0].optical_media[1]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        rem_pv_mock.assert_not_called()
        rem_pv_mock.reset_mock()
        rem_lv_mock.assert_not_called()
        rem_lv_mock.reset_mock()
        rem_lu_mock.assert_not_called()
        rem_lu_mock.reset_mock()
        rem_vopt_mock.assert_not_called()
        rem_vopt_mock.reset_mock()

        # pvmctl <obj_name> delete --lpar lpar_name --type lv \
        # --stor-id udid=abcd
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'lv', '--stor-id', "udid=abcd"]
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.virtual_disks[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        rem_pv_mock.assert_not_called()
        rem_pv_mock.reset_mock()
        rem_lv_mock.assert_not_called()
        rem_lv_mock.reset_mock()
        rem_lu_mock.assert_not_called()
        rem_lu_mock.reset_mock()
        rem_vopt_mock.assert_not_called()
        rem_vopt_mock.reset_mock()

        # pvmctl <obj_name> delete --lpar lpar_name --type pv \
        # --stor-id hdisk4
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk4']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        rem_pv_mock.assert_not_called()
        rem_pv_mock.reset_mock()
        rem_lv_mock.assert_not_called()
        rem_lv_mock.reset_mock()
        rem_lu_mock.assert_not_called()
        rem_lu_mock.reset_mock()
        rem_vopt_mock.assert_not_called()
        rem_vopt_mock.reset_mock()

        # pvmctl <obj_name> delete --lpar lpar_name --type pv \
        # --stor-id hdisk4 --vg name=image_pool
        args = ['scsi', 'delete', '--lpar', 'name=lpar_name', '--type',
                'pv', '--stor-id', 'hdisk4', '--vg', 'name=image_pool']
        cp = test_util.setup_mock(self, args, self.adpt,
                                  wrps=[vg_wrap.phys_vols[0]])

        self._mock_p_wrps(cp, vios_wrap)
        cp.obj_type.ec._add_parent_wrappers = mock.Mock(
            side_effect=self._mock_p_wrps(cp, vios_wrap))

        vg_search_mock.return_value = vg_wrap

        self.assertRaises(e.NoFoundDisksError, cp.obj_type.execute)

        rem_pv_mock.assert_not_called()
        rem_pv_mock.reset_mock()
        rem_lv_mock.assert_not_called()
        rem_lv_mock.reset_mock()
        rem_lu_mock.assert_not_called()
        rem_lu_mock.reset_mock()
        rem_vopt_mock.assert_not_called()
        rem_vopt_mock.reset_mock()
