# Licensed Materials - Property of IBM
#
# COPYRIGHT International Business Machines Corp. 2016
# All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.


import six
import sys

import pvmcli.util.exceptions as e
from pvmcli.util.i18n import _
import pvmcli.util.util as util

import pypowervm.wrappers.logical_partition as lpar_w
import pypowervm.wrappers.managed_system as msys_w
import pypowervm.wrappers.virtual_io_server as vios_w
import pypowervm.tests.test_utils.pvmhttp as pvmhttp


def execute(obj_type):
    ''' Build out pypowervm demo data on the provided object type and execute
        the user specified command against that demo payload.

        NOTE: Currently only list operations are supported.
    '''
    demo_file = None
    demo_wrp_cls = None
    if obj_type.model == msys_w.System:
        demo_file = 'managedsystem.txt'
        demo_wrp_cls = msys_w.System
    elif obj_type.model == lpar_w.LPAR:
        demo_file = 'lpar.txt'
        demo_wrp_cls = lpar_w.LPAR
    elif obj_type.model == vios_w.VIOS:
        demo_file = 'fake_vios.txt'
        demo_wrp_cls = vios_w.VIOS

    if demo_file:
        resp = pvmhttp.PVMResp(
            pvmfile=pvmhttp.PVMFile(demo_file),
            adapter=None).get_response()
        demo_wrps = demo_wrp_cls.wrap(resp)
        if not isinstance(demo_wrps, list):
            demo_wrps = [demo_wrps]
        obj_type.ec.wrappers = demo_wrps
        obj_type.ec.parsed_args.func()
    else:
        util.print_(_(
            'Demo not supported for %s object type.' % obj_type.name))


ERRORS = [sym for sym in [getattr(e, name) for name in dir(e)] if (
    issubclass(type(sym), type) and issubclass(sym, e.CLIAbstractMsgFmtError)
    and sym is not e.CLIAbstractMsgFmtError)]


def print_all_errors():
    for err_cls in ERRORS:
        err = err_cls(schema='demo', obj_type='demo', func_name='demo',
                      object_type='demo', pvid=1, name='demo',
                      operation='demo', storage_type='demo',
                      stor_id_val='demo', stor_id_type='demo',
                      vol_grps='demo', c_type='demo', p_type='demo',
                      auth_arg='demo', arg='demo', given_field='demo',
                      matches='demo', field='demo', id_arg='demo',
                      valid_fields='demo', logtype='demo', valid='demo',
                      level='demo', spp_id=1, proc_type='demo',
                      shr_modes='demo', loc='demo', spec_err='demo',
                      back_dev_str='demo', exp_pre='demo')
        util.print_('%s\n' % six.text_type(err), outfile=sys.stderr)
